import { useCallback } from 'react';
import { cancelAnimation, useAnimatedStyle, useSharedValue, withSpring, withTiming } from 'react-native-reanimated';
import { useDrag } from './useDrag';
import { emitter } from '../services/NotificationEmitter';
import { withAnimationCallbackJSThread } from '../utils/animation';
import { AnimationRange } from '../../types/animations';
import { useTimer } from './useTimer';

const mergeStylesObjects = (styles, newStyles) => {
  'worklet';

  const oldTransform = [...((styles === null || styles === void 0 ? void 0 : styles.transform) || [])];
  const newTransform = [...((newStyles === null || newStyles === void 0 ? void 0 : newStyles.transform) || [])];
  return { ...styles,
    ...newStyles,
    transform: [...oldTransform, ...newTransform]
  };
};

export const mergeStylesFunctions = (stylesFunctions, progress) => {
  'worklet';

  return stylesFunctions.reduce((accumulatedStyles, styleFunction) => {
    return mergeStylesObjects(accumulatedStyles, styleFunction(progress));
  }, {
    opacity: 1,
    transform: [{
      translateY: 0
    }, {
      translateX: 0
    }]
  } // it has to have the default opacity value
  );
};
export const useAnimationAPI = _ref => {
  let {
    gestureConfig,
    animationConfig,
    duration,
    onClose
  } = _ref;
  const progress = useSharedValue(0);
  const {
    resetTimer,
    clearTimer
  } = useTimer();
  const animationInConfig = animationConfig.animationConfigIn;
  const animationOutConfig = animationConfig === null || animationConfig === void 0 ? void 0 : animationConfig.animationConfigOut;
  const {
    dragStateHandler,
    resetDrag,
    ...dragConfig
  } = useDrag(gestureConfig);
  const currentTransitionType = useSharedValue('in');
  const dismiss = useCallback(id => {
    currentTransitionType.value = 'out';
    clearTimer();
    resetDrag();
    const dismissConfig = animationOutConfig || animationInConfig;
    const animateWith = dismissConfig.type === 'spring' ? withSpring : withTiming;

    const handleSuccess = () => {
      currentTransitionType.value = 'in';
      emitter.emit('pop_notification', id);
      onClose === null || onClose === void 0 ? void 0 : onClose();
    };

    const handleError = () => {};

    progress.value = animateWith(AnimationRange.END, dismissConfig.config, withAnimationCallbackJSThread(handleSuccess, handleError));
  }, [currentTransitionType, clearTimer, resetDrag, animationOutConfig, animationInConfig, progress, onClose]);
  const present = useCallback(() => {
    currentTransitionType.value = 'in';
    const animateWith = animationInConfig.type === 'spring' ? withSpring : withTiming;

    const handleSuccess = () => {
      currentTransitionType.value = 'idle_active';
      resetTimer(dismiss, duration);
    };

    const handleError = () => {};

    progress.value = animateWith(AnimationRange.START, animationInConfig.config, withAnimationCallbackJSThread(handleSuccess, handleError));
  }, [animationInConfig, currentTransitionType, dismiss, duration, progress, resetTimer]);
  const cancelTransitionAnimation = useCallback(() => {
    clearTimer();
    cancelAnimation(progress);
  }, [clearTimer, progress]); // Used to revoke transition (progress) value after canceling it with e.g. LongPressGestureHandler

  const revokeTransitionAnimation = useCallback(() => {
    switch (currentTransitionType.value) {
      case 'out':
        return dismiss();

      case 'in':
      case 'idle_active':
        return resetTimer(dismiss, duration);
    }
  }, [currentTransitionType.value, dismiss, resetTimer, duration]);
  const handleDragStateChange = dragStateHandler(dismiss, resetDrag);
  const {
    transitionInStyles,
    transitionOutStyles,
    transitionInStylesQueue,
    transitionOutStylesQueue
  } = animationConfig;
  const animatedStyles = useAnimatedStyle(() => {
    if (['out', 'idle_active'].includes(currentTransitionType.value) && (transitionOutStylesQueue === null || transitionOutStylesQueue === void 0 ? void 0 : transitionOutStylesQueue.length) > 0) {
      return mergeStylesFunctions(transitionOutStylesQueue, progress);
    }

    if ((transitionInStylesQueue === null || transitionInStylesQueue === void 0 ? void 0 : transitionInStylesQueue.length) > 0) {
      return mergeStylesFunctions(transitionInStylesQueue, progress);
    }

    if (['out', 'idle_active'].includes(currentTransitionType.value) && transitionOutStyles) {
      return {
        opacity: 1,
        ...transitionOutStyles(progress)
      };
    }

    return {
      opacity: 1,
      ...transitionInStyles(progress)
    };
  });
  return { ...dragConfig,
    present,
    dismiss,
    animatedStyles,
    handleDragStateChange,
    cancelTransitionAnimation,
    revokeTransitionAnimation
  };
};
//# sourceMappingURL=useAnimationAPI.js.map